import React, { useState, useEffect } from 'react'
import fetch from 'unfetch'

import SubjectView from '../components/subjectView'
import QuestionView from '../components/questionView'
import QuestionAdder from '../components/questionAdder.js'
import LoadingIndicator from '../components/LoadingIndicator'

import styles from './index.module.css'
import constants from '../constants.json'

const views = {
  welcome: 'WELCOME',
  subject: 'SUBJECT',
  question: 'QUESTION',
  questionAdder: 'QADDER',
}

// TODO: Add question on subjects view
// TODO: Add subject on subjects view
// TODO: question.data editor
// TODO: edit \n-s in questions / answers

export default function Index() {
  const [data, setData] = useState(null)
  const [qdbs, setQdbs] = useState(null)
  const [selectedDb, setSelectedDb] = useState()
  const [view, setView] = useState(views.welcome)
  const [sending, setSending] = useState(false)
  const [error, setError] = useState(null)
  const [password, setPassword] = useState('')
  const [editedQuestions, setEditedQuestions] = useState({})

  const [initialCount, setInitialCount] = useState({})

  useEffect(() => {
    if (selectedDb) {
      loadData()
    }
  }, [selectedDb])

  const getCount = (data) => {
    return {
      subjectCount: data.length,
      questionCount: data.reduce((acc, subj) => {
        acc += subj.Questions.length
        return acc
      }, 0),
    }
  }

  const setIndexes = (data) => {
    data.forEach((subj, i) => {
      subj.ind = i
      subj.Questions.forEach((question, j) => {
        question.ind = j
      })
    })
    return data
  }

  const loadData = () => {
    setData(null)
    if (!selectedDb) {
      alert('Válassz egy adatbázist!')
      return
    }

    const toFetch = `${constants.apiUrl}${selectedDb.path}`
    console.info('Fetching', toFetch)
    fetch(toFetch, {
      credentials: 'include',
    })
      .then((resp) => {
        return resp.json()
      })
      .then((resp) => {
        setData(setIndexes(resp))
        const count = getCount(resp)
        setInitialCount(count)
      })
      .catch((error) => {
        console.error(error)
        console.error('Error while fetching data')
        setError('Error while fetching data')
      })
  }

  useEffect(() => {
    fetch(`${constants.apiUrl}getdbs`, {
      credentials: 'include',
    })
      .then((resp) => {
        return resp.json()
      })
      .then((resp) => {
        setQdbs(resp)
      })
  }, [])

  if (error) {
    return <div>{error}</div>
  }

  const deleteQuestion = (subjInd, questionInd) => {
    data[subjInd].Questions.splice(questionInd, 1)

    setData([...setIndexes(data)])
  }

  const onChange = (subjInd, questionInd, newVal) => {
    const key = subjInd + '/' + questionInd
    setEditedQuestions({
      ...editedQuestions,
      [key]: editedQuestions[key] ? editedQuestions[key] + 1 : 1,
    })

    data[subjInd].Questions[questionInd] = newVal
    setData([...data])
  }

  const SendDataToServer = async () => {
    if (sending) {
      alert('Adatok már feltöltés alatt, várd meg míg feltölti') // eslint-disable-line
      return
    }

    data.forEach((subj) => {
      delete subj.ind
      subj.Questions.forEach((question) => {
        delete question.ind
      })
    })

    setSending(true)
    try {
      const rawResponse = await fetch(constants.apiUrl + 'uploaddata', {
        method: 'POST',
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json',
        },
        credentials: 'include',
        body: JSON.stringify({
          newData: data,
          count: getCount(data),
          initialCount: initialCount,
          password: password,
          editedQuestions: editedQuestions,
        }),
      })
      rawResponse
        .json()
        .then((resp) => {
          setSending(false)
          if (resp.status === 'ok') {
            alert(`Sikeres feltöltés! thankx ${resp.user}!`) // eslint-disable-line
            console.log('OK')
            loadData()
          } else if (resp.status === 'invalidPass') {
            alert('Hibás jelszó!') // eslint-disable-line
            console.log('invalidPass')
          } else {
            alert(
              'Hiba feltöltés közben! (szerver oldalon)! Több adat konzolban'
            ) // eslint-disable-line
            console.error('RESPONSE', resp)
            console.error(resp.message)
          }
        })
        .catch((error) => {
          setSending(false)
          alert('Hiba feltöltés közben! (kliens oldalon)! Több adat konzolban') // eslint-disable-line
          console.error('Error posting data', error)
        })
    } catch (error) {
      setSending(false)
      alert('Hiba feltöltés közben! (kliens oldalon)! Több adat konzolban') // eslint-disable-line
      console.error('Error posting data', error)
    }
  }

  const renderView = () => {
    if (view === views.subject) {
      return (
        <SubjectView
          onChange={onChange}
          data={data}
          deleteQuestion={deleteQuestion}
        />
      )
    } else if (view === views.question) {
      return (
        <QuestionView
          onChange={onChange}
          data={data}
          deleteQuestion={deleteQuestion}
        />
      )
    } else if (view === views.questionAdder) {
      return <QuestionAdder />
    } else if (view === views.welcome) {
      return (
        <div className={styles.welcome}>
          Kérdés szerkesztő
          <br />
          {initialCount.questionCount || '...'} kérdés,{' '}
          {initialCount.subjectCount || '...'} tárgy
          <p />
          Itt az éles adatbázis kérdései jelennek meg, amiket tudsz
          szerkeszteni. A kérdésekhez tartozó .data prop-ot még nem tudod
          rendesen szerkeszteni, az később lesz implementálva. A Tárgy / Kérdés
          nézet között tudsz válogatni.
          <br />
          Tárgy nézet: a tárgyak dobozban kattints egy tárgyra, minden kérdés
          ott lessz hozzá.
          <br />
          Kérdés nézet: Kereső sávba írd be a kérdést / választ, a találatok ha
          kevesebb mint 250 megjelennek, és ott tudod szerkeszteni
        </div>
      )
    } else {
      return <div>No view!</div>
    }
  }

  const renderDbSelector = () => {
    return (
      <>
        <select
          style={{ margin: '10px 0px' }}
          defaultValue={-1}
          onChange={(event) => {
            setSelectedDb(qdbs[event.target.value])
          }}
        >
          <option disabled value={-1}>
            {' -- Válassz egy kérdés adatbázist -- '}
          </option>
          {qdbs.map((qdb, i) => {
            return (
              <option value={i} key={qdb.name}>
                {qdb.name}
              </option>
            )
          })}
        </select>
      </>
    )
  }

  if (!qdbs) {
    return <LoadingIndicator />
  }

  return (
    <div>
      {renderDbSelector()}
      <div className={styles.optionsButtonContainer}>
        <span
          onClick={() => {
            loadData()
          }}
        >
          Újratöltés
        </span>
        <span>
          <input
            placeholder="Jelszó kérdések módosításhoz"
            type="text"
            value={password}
            onChange={(event) => {
              setPassword(event.target.value)
            }}
          />
        </span>
        <span
          onClick={() => {
            if (password && selectedDb) {
              SendDataToServer()
            } else {
              alert('Nincs jelszó, vagy nem választottál ki adatbázist!') // eslint-disable-line
            }
          }}
        >
          {sending ? 'Feltöltés ...' : 'Mentés'}
        </span>
      </div>
      <div className={styles.viewButtonContainer}>
        <span
          onClick={() => {
            if (selectedDb) {
              setView(views.question)
            } else {
              alert('Válassz egy adatbázist!')
            }
          }}
        >
          Kérdés nézet
        </span>
        <span
          title={
            'Választott adatbázisban lévő tárgyak megjelenítése, és tárgyakon belüli kérdések szerkesztése'
          }
          onClick={() => {
            if (selectedDb) {
              setView(views.subject)
            } else {
              alert('Válassz egy adatbázist!')
            }
          }}
        >
          Tárgy nézet
        </span>
        <span
          onClick={() => {
            setView(views.questionAdder)
          }}
        >
          Kérdés beküldés
        </span>
      </div>
      {renderView()}
    </div>
  )
}
